@@ -67,6 +67,8 @@ from .frdata import *
 from .canonical import *
 from .robust import *
 from .config import *
+from .sisotool import *
+from .iosys import *
 
 # Exceptions
 from .exception import *
@@ -82,3 +84,6 @@ except ImportError:
 from numpy.testing import Tester
 test = Tester().test
 bench = Tester().bench
+
+# Initialize default parameter values
+reset_defaults()
