@@ -10,7 +10,6 @@
 
 import unittest
 import numpy as np
-# import scipy as sp
 from control.timeresp import *
 from control.statesp import *
 from control.xferfcn import TransferFunction, _convert_to_transfer_function
@@ -253,14 +252,14 @@ class TestTimeresp(unittest.TestCase):
         # first system: initial value, second system: step response
         u = np.array([[0., 0, 0, 0, 0, 0, 0, 0, 0, 0],
                       [1., 1, 1, 1, 1, 1, 1, 1, 1, 1]])
-        x0 = np.matrix(".5; 1; 0; 0")
+        x0 = np.array([[.5], [1], [0], [0]])
         youttrue = np.array([[11., 8.1494, 5.9361, 4.2258, 2.9118, 1.9092,
                               1.1508, 0.5833, 0.1645, -0.1391],
                              [9., 17.6457, 24.7072, 30.4855, 35.2234, 39.1165,
                               42.3227, 44.9694, 47.1599, 48.9776]])
         _t, yout, _xout = forced_response(self.mimo_ss1, t, u, x0)
         np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
-        
+
         # Test discrete MIMO system to use correct convention for input
         sysc = self.mimo_ss1
         dt=t[1]-t[0]
@@ -271,6 +270,17 @@ class TestTimeresp(unittest.TestCase):
         np.testing.assert_array_equal(youtc.shape, youtd.shape)
         np.testing.assert_array_almost_equal(youtc, youtd, decimal=4)
 
+        # Test discrete MIMO system without default T argument
+        u = np.array([[0., 0, 0, 0, 0, 0, 0, 0, 0, 0],
+                      [1., 1, 1, 1, 1, 1, 1, 1, 1, 1]])
+        x0 = np.array([[.5], [1], [0], [0]])
+        youttrue = np.array([[11., 8.1494, 5.9361, 4.2258, 2.9118, 1.9092,
+                              1.1508, 0.5833, 0.1645, -0.1391],
+                             [9., 17.6457, 24.7072, 30.4855, 35.2234, 39.1165,
+                              42.3227, 44.9694, 47.1599, 48.9776]])
+        _t, yout, _xout = forced_response(sysd, U=u, X0=x0)
+        np.testing.assert_array_almost_equal(yout, youttrue, decimal=4)
+
     def test_lsim_double_integrator(self):
         # Note: scipy.signal.lsim fails if A is not invertible
         A = np.mat("0. 1.;0. 0.")
@@ -309,7 +319,7 @@ class TestTimeresp(unittest.TestCase):
     def test_discrete_initial(self):
         h1 = TransferFunction([1.], [1., 0.], 1.)
         t, yout = impulse_response(h1, np.arange(4))
-        np.testing.assert_array_equal(yout[0], [0., 1., 0., 0.])
+        np.testing.assert_array_equal(yout, [0., 1., 0., 0.])
 
     @unittest.skipIf(not slycot_check(), "slycot not installed")
     def test_step_robustness(self):
@@ -345,27 +355,32 @@ class TestTimeresp(unittest.TestCase):
         # No timebase in system => output should match input
         #
         # Initial response
-        tout, yout = initial_response(self.siso_dtf1, Tin2, siso_x0)
+        tout, yout = initial_response(self.siso_dtf1, Tin2, siso_x0,
+                                      squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin2)
 
         # Impulse response
-        tout, yout = impulse_response(self.siso_dtf1, Tin2)
+        tout, yout = impulse_response(self.siso_dtf1, Tin2,
+                                      squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin2)
 
         # Step response
-        tout, yout = step_response(self.siso_dtf1, Tin2)
+        tout, yout = step_response(self.siso_dtf1, Tin2,
+                                   squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin2)
 
         # Forced response with specified time vector
-        tout, yout, xout = forced_response(self.siso_dtf1, Tin2, np.sin(Tin2))
+        tout, yout, xout = forced_response(self.siso_dtf1, Tin2, np.sin(Tin2),
+                                           squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin2)
 
         # Forced response with no time vector, no sample time (should use 1)
-        tout, yout, xout = forced_response(self.siso_dtf1, None, np.sin(Tin1))
+        tout, yout, xout = forced_response(self.siso_dtf1, None, np.sin(Tin1),
+                                           squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin1)
 
@@ -380,49 +395,58 @@ class TestTimeresp(unittest.TestCase):
         # Matching timebase in system => output should match input
         #
         # Initial response
-        tout, yout = initial_response(self.siso_dtf2, Tin2, siso_x0)
+        tout, yout = initial_response(self.siso_dtf2, Tin2, siso_x0,
+                                      squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin2)
 
         # Impulse response
-        tout, yout = impulse_response(self.siso_dtf2, Tin2)
+        tout, yout = impulse_response(self.siso_dtf2, Tin2,
+                                      squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin2)
 
         # Step response
-        tout, yout = step_response(self.siso_dtf2, Tin2)
+        tout, yout = step_response(self.siso_dtf2, Tin2,
+                                   squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin2)
 
         # Forced response
-        tout, yout, xout = forced_response(self.siso_dtf2, Tin2, np.sin(Tin2))
+        tout, yout, xout = forced_response(self.siso_dtf2, Tin2, np.sin(Tin2),
+                                           squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin2)
 
         # Forced response with no time vector, use sample time
-        tout, yout, xout = forced_response(self.siso_dtf2, None, np.sin(Tin2))
+        tout, yout, xout = forced_response(self.siso_dtf2, None, np.sin(Tin2),
+                                           squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin2)
 
         # Compatible timebase in system => output should match input
         #
         # Initial response
-        tout, yout = initial_response(self.siso_dtf2, Tin1, siso_x0)
+        tout, yout = initial_response(self.siso_dtf2, Tin1, siso_x0,
+                                      squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin1)
 
         # Impulse response
-        tout, yout = impulse_response(self.siso_dtf2, Tin1)
+        tout, yout = impulse_response(self.siso_dtf2, Tin1,
+                                      squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin1)
 
         # Step response
-        tout, yout = step_response(self.siso_dtf2, Tin1)
+        tout, yout = step_response(self.siso_dtf2, Tin1,
+                                   squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin1)
 
         # Forced response
-        tout, yout, xout = forced_response(self.siso_dtf2, Tin1, np.sin(Tin1))
+        tout, yout, xout = forced_response(self.siso_dtf2, Tin1, np.sin(Tin1),
+                                           squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         np.testing.assert_array_equal(tout, Tin1)
 
@@ -431,7 +455,8 @@ class TestTimeresp(unittest.TestCase):
         #
         # Initial response
         tout, yout, xout = forced_response(self.siso_dtf2, Tin1,
-                                           np.sin(Tin1), interpolate=True)
+                                           np.sin(Tin1), interpolate=True,
+                                           squeeze=False)
         self.assertEqual(np.shape(tout), np.shape(yout[0,:]))
         self.assertTrue(np.allclose(tout[1:] - tout[:-1],  self.siso_dtf2.dt))
 
@@ -442,9 +467,101 @@ class TestTimeresp(unittest.TestCase):
         #
         # Initial response
         with self.assertRaises(Exception) as context:
-            tout, yout = initial_response(self.siso_dtf2, Tin3, siso_x0)
+            tout, yout = initial_response(self.siso_dtf2, Tin3, siso_x0,
+                                          squeeze=False)
         self.assertTrue(isinstance(context.exception, ValueError))
 
+    def test_discrete_time_steps(self):
+        """Make sure rounding errors in sample time are handled properly"""
+        # See https://github.com/python-control/python-control/issues/332)
+        #
+        # These tests play around with the input time vector to make sure that
+        # small rounding errors don't generate spurious errors.
+
+        # Discrete time system to use for simulation
+        # self.siso_dtf2 = TransferFunction([1], [1, 1, 0.25], 0.2)
+
+        # Set up a time range and simulate
+        T = np.arange(0, 100, 0.2)
+        tout1, yout1 = step_response(self.siso_dtf2, T)
+
+        # Simulate every other time step
+        T = np.arange(0, 100, 0.4)
+        tout2, yout2 = step_response(self.siso_dtf2, T)
+        np.testing.assert_array_almost_equal(tout1[::2], tout2)
+        np.testing.assert_array_almost_equal(yout1[::2], yout2)
+
+        # Add a small error into some of the time steps
+        T = np.arange(0, 100, 0.2)
+        T[1:-2:2] -= 1e-12      # tweak second value and a few others
+        tout3, yout3 = step_response(self.siso_dtf2, T)
+        np.testing.assert_array_almost_equal(tout1, tout3)
+        np.testing.assert_array_almost_equal(yout1, yout3)
+
+        # Add a small error into some of the time steps (w/ skipping)
+        T = np.arange(0, 100, 0.4)
+        T[1:-2:2] -= 1e-12      # tweak second value and a few others
+        tout4, yout4 = step_response(self.siso_dtf2, T)
+        np.testing.assert_array_almost_equal(tout2, tout4)
+        np.testing.assert_array_almost_equal(yout2, yout4)
+
+        # Make sure larger errors *do* generate an error
+        T = np.arange(0, 100, 0.2)
+        T[1:-2:2] -= 1e-3      # change second value and a few others
+        self.assertRaises(ValueError, step_response, self.siso_dtf2, T)
+
+    def test_time_series_data_convention(self):
+        """Make sure time series data matches documentation conventions"""
+        # SISO continuous time
+        t, y = step_response(self.siso_ss1)
+        self.assertTrue(isinstance(t, np.ndarray)
+                        and not isinstance(t, np.matrix))
+        self.assertTrue(len(t.shape) == 1)
+        self.assertTrue(len(y.shape) == 1) # SISO returns "scalar" output
+        self.assertTrue(len(t) == len(y))  # Allows direct plotting of output
+
+        # SISO discrete time
+        t, y = step_response(self.siso_dss1)
+        self.assertTrue(isinstance(t, np.ndarray)
+                        and not isinstance(t, np.matrix))
+        self.assertTrue(len(t.shape) == 1)
+        self.assertTrue(len(y.shape) == 1) # SISO returns "scalar" output
+        self.assertTrue(len(t) == len(y))  # Allows direct plotting of output
+
+        # MIMO continuous time
+        tin = np.linspace(0, 10, 100)
+        uin = [np.sin(tin), np.cos(tin)]
+        t, y, x = forced_response(self.mimo_ss1, tin, uin)
+        self.assertTrue(isinstance(t, np.ndarray)
+                        and not isinstance(t, np.matrix))
+        self.assertTrue(len(t.shape) == 1)
+        self.assertTrue(len(y[0].shape) == 1)
+        self.assertTrue(len(y[1].shape) == 1)
+        self.assertTrue(len(t) == len(y[0]))
+        self.assertTrue(len(t) == len(y[1]))
+
+        # MIMO discrete time
+        tin = np.linspace(0, 10, 100)
+        uin = [np.sin(tin), np.cos(tin)]
+        t, y, x = forced_response(self.mimo_dss1, tin, uin)
+        self.assertTrue(isinstance(t, np.ndarray)
+                        and not isinstance(t, np.matrix))
+        self.assertTrue(len(t.shape) == 1)
+        self.assertTrue(len(y[0].shape) == 1)
+        self.assertTrue(len(y[1].shape) == 1)
+        self.assertTrue(len(t) == len(y[0]))
+        self.assertTrue(len(t) == len(y[1]))
+
+        # Allow input time as 2D array (output should be 1D)
+        tin = np.array(np.linspace(0, 10, 100), ndmin=2)
+        t, y = step_response(self.siso_ss1, tin)
+        self.assertTrue(isinstance(t, np.ndarray)
+                        and not isinstance(t, np.matrix))
+        self.assertTrue(len(t.shape) == 1)
+        self.assertTrue(len(y.shape) == 1) # SISO returns "scalar" output
+        self.assertTrue(len(t) == len(y))  # Allows direct plotting of output
+
+
 def suite():
     return unittest.TestLoader().loadTestsFromTestCase(TestTimeresp)
 
